/*******************************************************************
*
*  DESCRIPTION: Buzzer Driver Atomic Model Header File 
*
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 07 October 2001
*
*******************************************************************/

#ifndef __BUZDRV_H
#define __BUZDRV_H

#include <list>
#include "atomic.h"     // class Atomic

#include "VTime.hh"

// BuzDrv class

class BuzDrv : public Atomic
{
public:
	// Constructor
	BuzDrv( const string &name = "BuzDrv" );

	virtual string className() const ;
protected:
	Model &initFunction();
	Model &externalFunction( const ExternalMessage & );
	Model &internalFunction( const InternalMessage & );
//	Model &outputFunction( const InternalMessage & );
	Model &outputFunction( const CollectMessage & );

private:

	// input ports
	const Port &time_set;
	const Port &alarm_set;
	const Port &time_match;
	const Port &alarm_on;
	const Port &snooze;

	// output port
	Port &buzzer_on;

	
	// some constants
	const VTime one_hour;
	const VTime ten_minutes;
	

	// time buzzer must be turned off 
	VTime timeRemaining;

	// time when buzzer was turned on
	VTime alarmTime;

	// state variables
	bool armed;	
	bool alarmSet;
	bool timeSet;
	bool snooz;
	bool buzzer;


};	// class BuzDrv

// ** inline ** // 
inline
string BuzDrv::className() const
{
	return "BuzDrv" ;
}

#endif   //__BUZDRV_H
