/*******************************************************************
*
*  DESCRIPTION: Time Comparator Atomic Model
*
*  This is the implementation of the Time Comparator Atomic Model.
*  Its purpose is to compare two values of time, tipically a time 
*  of day and an an alarm time, and provide an indication when
*  the two times match.  The model has two inputs and one output 
*  defined as follows:
*
*  <time_of_day>: This input provides the time_of_day.
*
*  <alarm_time>: This input provides the alarm time.
*
*  <time_match>: A value of true is sent to this output port
*  when the values received on <time_of_day> and <alarm_time> 
*  match.
*
*  AUTHOR: Christian Jacques
*
*  EMAIL: chris.jacques@videotron.ca
*
*  DATE: 05 October 2001
*
*******************************************************************/

/** include files **/
#include "timeComparator.h"  // class TimeComp
#include "message.h"    // class ExternalMessage, InternalMessage
//#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include "parsimu.h"

/** public functions **/

/*******************************************************************
* Function Name: TimeComp constructor
* Description: This routine constructs the TimeComp model
********************************************************************/
TimeComp::TimeComp( const string &name )
: Atomic( name )
, alarm_time( addInputPort( "alarm_time" ) )
, time_of_day( addInputPort( "time_of_day" ) )
, time_match( addOutputPort( "time_match" ) )
{
}

/*******************************************************************
* Function Name: TimeComp init function
* Description: This routine is invoked when simulation starts.
* This method does nothing.
********************************************************************/
Model &TimeComp::initFunction()
{
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: This method handles external events coming from
* any of the two input ports.
********************************************************************/
Model &TimeComp::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == alarm_time )
	   {
	   this->alarmTime = msg.value();
	   }
	else // the port is time_of_day
	   {
	   this->timeOfDay = msg.value();
	   }
	
	if( timeOfDay == alarmTime )
	   holdIn( AtomicState::passive, VTime::Zero );

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: The internal function simply passivates the model
********************************************************************/
Model &TimeComp::internalFunction( const InternalMessage & )
{
	passivate ();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
* Description: This routines send the value of the time match
* variable to the <time_match> port.
********************************************************************/
//Model &TimeComp::outputFunction( const InternalMessage &msg )
Model &TimeComp::outputFunction( const CollectMessage &msg )
{
	sendOutput( msg.time(), time_match, this->alarmTime );
	return *this ;
}
